@@ -5,10 +5,10 @@ enforces GeoJSON standards. """
 import copy
 import json
 from collections import namedtuple
-import itertools
 from numbers import Number
 from math import isnan
 from functools import reduce
+from ..crs import CRS
 
 Point = namedtuple('Point', ['coordinates', 'crs'])
 MultiPoint = namedtuple('MultiPoint', ['coordinates', 'crs'])
@@ -16,14 +16,31 @@ LineString = namedtuple('LineString', ['coordinates', 'crs'])
 MultiLineString = namedtuple('MultiLineString', ['coordinates', 'crs'])
 Polygon = namedtuple('Polygon', ['coordinates', 'crs'])
 MultiPolygon = namedtuple('MultiPolygon', ['coordinates', 'crs'])
-GeometryCollection = namedtuple('GeometryCollection', ['geometries'])
+GeometryCollection = namedtuple('GeometryCollection', ['geometries', 'crs'])
 Feature = namedtuple('Feature', ['geometry', 'properties', 'id', 'crs'])
 FeatureCollection = namedtuple('FeatureCollection', ['features', 'crs'])
 
-DEFAULTCRS = {'type': 'name',
-              'properties': {'name': 'urn:ogc:def:crs:EPSG::4326'}}
+class GeoJSONNamedCRS(CRS):
+    def __init__(self, name):
+        self.name = name
+        self.jsonname = name
 
-class ExtendedJSONEncoder(json.JSONEncoder):
+    def __eq__(self, other):
+        return self.jsonname == other.jsonname
+
+class GeoJSONLinkedCRS(CRS):
+    def __init__(self, href, linktype):
+        self.name = href
+        self.jsonhref = href
+        self.jsontype = linktype
+
+    def __eq__(self, other):
+        return self.jsonhref == other.jsonhref
+
+DEFAULTCRS = {"type": "name",
+              "properties": {"name": "urn:ogc:def:crs:OGC:1.3:CRS84"}}
+
+class NumpyAwareJSONEncoder(json.JSONEncoder):
     """ Numpy-specific numbers prior to 1.9 don't inherit from Python numeric
     ABCs. This class is a hack to coerce numpy values into Python types for
     JSON serialization. """
@@ -43,18 +60,15 @@ class ExtendedJSONEncoder(json.JSONEncoder):
 
 
 class GeoJSONWriter(object):
-    """ Class for converting guppy objects to GeoJSON strings. Multipoint-based
-    opbjects are written as 'Features'.
-
-    CRS defaults to be named. A linked CRS can be used by passing
-    `linkedcrs=True`.
+    """ Class for converting geometry objects to GeoJSON strings. Multipoint-based
+    opjects are written as 'Features'.
 
     Notes:
     ------
     Does not handle 'FeatureCollection' types (see printFeatureCollection()
     function).
     """
-    def __init__(self, gpobj, linkedcrs=False, **kwargs):
+    def __init__(self, gpobj, **kwargs):
         type_equiv = {'Point'       : 'Point',
                       'Multipoint'  : 'MultiPoint',
                       'Line'        : 'LineString',
@@ -64,15 +78,23 @@ class GeoJSONWriter(object):
             self.typestr = type_equiv[gpobj._geotype]
         else:
             raise TypeError('Input object not a recognized geometry')
-        crs = gpobj._crs
-        bbox = kwargs.get('bbox', None)
+
 
         self.gpobj = gpobj
         self.supobj = {}
-        if crs is not None:
-            self.add_crs(crs, linkedcrs=linkedcrs)
+
+        self.crs = None     # Becomes not None if a non-native CRS is added
+
+        if 'urn' in kwargs:
+            self.add_crs(urn=kwargs['urn'])
+        elif gpobj.crs is None:
+            self.add_crs(crs=DEFAULTCRS)
+        else:
+            self.add_crs(crs=gpobj.crs)
+
         if self.typestr != 'Point':
             self.supobj['type'] = 'Feature'
+            bbox = kwargs.get('bbox', None)
             self.add_bbox(bbox)
             self.add_geometry()
         else:
@@ -80,30 +102,34 @@ class GeoJSONWriter(object):
             self.add_coordinates()
         return
 
-    def add_crs(self, crs, linkedcrs, **kw):
+    def add_crs(self, crs=None, urn=None):
         """ Tag the JSON object with coordinate reference system metadata using
         a CRS instance, which is expected the have the interface of crs.CRS.
 
-        If True, `linkedcrs` indicates a web-referenced linked CRS. If False,
-        the CRS is taked to be named.
-
         In the case of a linked CRS, the link address and type can be specified
         using the `href` and `linktype` keyword arguments. If not supplied,
-        then it is assumed that crs.urn contains the *href* and crs.type
-        contains the *type*.
+        then it is assumed that crs.id["urn"] contains the *href* and the
+        *type* is "name".
 
         For more details, see the GeoJSON specification at:
         http://geojson.org/geojson-spec.html#coordinate-reference-system-objects
         """
-        if linkedcrs:
-            href = kw.get('href', crs.urn)
-            linktype = kw.get('linktype', crs.type)
-            self.supobj['crs'] = {'type': 'link',
-                                  'properties': {'href': href, 'type': linktype}}
-
-        else:
+        if urn is not None:
             self.supobj['crs'] = {'type': 'name',
-                                  'properties': {'name': crs.urn}}
+                                  'properties': {'name': urn}}
+        else:
+            if hasattr(crs, "jsonhref") and hasattr(crs, "jsontype"):
+                self.supobj['crs'] = {'type': 'link',
+                                      'properties': {'href': crs.jsonname,
+                                                     'type': crs.jsontype}}
+            elif hasattr(crs, "jsonname"):
+                self.supobj['crs'] = {'type': 'name',
+                                      'properties': {'name': crs.jsonname}}
+            else:
+                self.crs = crs
+                self.supobj['crs'] = {'type': 'link',
+                                      'properties': {'href': "",
+                                                     'type': 'proj4'}}
         return
 
     def add_bbox(self, bbox=None):
@@ -112,10 +138,11 @@ class GeoJSONWriter(object):
         *bbox* should be in the form (xmin, xmax, ....) for all dimensions.
         """
         if bbox is None:
-            if hasattr(self.gpobj, 'get_extents'):
-                bbox = self.gpobj.get_extents()
+            if hasattr(self.gpobj, 'get_extent'):
+                xmn, xmx, ymn, ymx = self.gpobj.get_extent()
+                bbox = ((xmn, xmx), (ymn, ymx))
             else:
-                raise AttributeError("Type {0} has no 'get_extents' "
+                raise AttributeError("Type {0} has no 'get_extent' "
                                      "method".format(type(self.gpobj)))
         self.supobj['bbox'] = bbox
         return
@@ -137,14 +164,14 @@ class GeoJSONWriter(object):
             target = self.supobj
 
         if self.gpobj._geotype == 'Polygon':
-            target['coordinates'] = [list_rec(self.gpobj.get_vertices())]
+            target['coordinates'] = [asfixedlist(self.gpobj.get_vertices())]
             if hasattr(self.gpobj, "subs"):
                 for poly in self.gpobj.subs:
-                    target['coordinates'].append(list_rec(poly.get_vertices()))
+                    target['coordinates'].append(asfixedlist(poly.get_vertices()))
         elif hasattr(self.gpobj, 'get_vertices'):
-            target['coordinates'] = list_rec(self.gpobj.get_vertices())
+            target['coordinates'] = asfixedlist(self.gpobj.get_vertices())
         elif hasattr(self.gpobj, 'get_vertex'):
-            target['coordinates'] = list_rec(self.gpobj.get_vertex())
+            target['coordinates'] = asfixedlist(self.gpobj.get_vertex())
         else:
             raise AttributeError('Geometry object has no vertex method')
         return
@@ -154,14 +181,15 @@ class GeoJSONWriter(object):
         if target is None:
             target = self.supobj
         target['properties'] = {}
-        if (False in (a is None for a in self.gpobj.data)):
-            if hasattr(self.gpobj.data, 'keys'):
+        if self.gpobj.data is not None:
+            if hasattr(self.gpobj.data, 'fields'):
                 data = self.gpobj.data
+                data = dict((f, data.getfield(f)) for f in data.fields)
             else:
                 data = {'point_data': self.gpobj.data}
             for key in data:
                 nan2none = lambda a: None if isinstance(a, Number) and isnan(a) else a
-                target['properties'][key] = list(map(nan2none, data.getfield(key)))
+                target['properties'][key] = list(map(nan2none, data[key]))
         return
 
     def add_id(self, target=None):
@@ -171,95 +199,106 @@ class GeoJSONWriter(object):
         target['id'] = list(range(len(self.gpobj.get_vertices())))
         return
 
-    def print_json(self):
+    def print_json(self, indent):
         """ Print GeoJSON representation. """
-        return json.dumps(self.supobj, indent=2, cls=ExtendedJSONEncoder)
+        return json.dumps(self.supobj, indent=indent, cls=NumpyAwareJSONEncoder)
 
-    def write_json(self, fout):
+    def write_json(self, fout, indent):
         """ Dump internal dict-object to JSON using the builtin `json` module.
         """
-        json.dump(self.supobj, fout, indent=2, cls=ExtendedJSONEncoder)
+        if self.crs is not None:
+            # Indicates that a CRS object should be used to create a linked CRS
+            fout_crs = fout.name+".proj4"
+            self.supobj["crs"]["properties"]["href"] = fout_crs
+            with open(fout_crs, "w") as f:
+                f.write(self.crs.get_proj4())
+        json.dump(self.supobj, fout, indent=indent, cls=NumpyAwareJSONEncoder)
         return
 
 class GeoJSONReader(object):
 
-    def __init__(self, finput):
+    def __init__(self, finput, defaultcrs=DEFAULTCRS):
         """ Create a reader-object for a GeoJSON-containing file or StreamIO
         object. Use as::
 
             with open(`fnm`, 'r') as f:
                 reader = GeoJSONReader(f)
         """
-        if not hasattr(finput, 'read'):
-            with open(finput) as f:
-                self.jsondict = json.load(f)
-        else:
+        if hasattr(finput, 'read'):
             self.jsondict = json.load(finput)
+        elif isinstance(finput, str):
+            try:
+                self.jsondict = json.loads(finput)
+            except ValueError:
+                with open(finput) as f:
+                    self.jsondict = json.load(f)
+        else:
+            raise TypeError("*finput* must be a file object, a JSON string, or "
+                            "a filename string")
+
+        self.defaultcrs = defaultcrs
         return
 
-    @staticmethod
-    def parsePoint(d, defaultcrs):
-        crs = d.get("crs", defaultcrs)
+    def parsePoint(self, d):
+        crs = d.get("crs", self.defaultcrs)
         return Point(d["coordinates"], crs)
 
-    @staticmethod
-    def parseMultiPoint(d, defaultcrs):
-        crs = d.get("crs", defaultcrs)
+    def parseMultiPoint(self, d):
+        crs = d.get("crs", self.defaultcrs)
         return MultiPoint(d["coordinates"], crs)
 
-    @staticmethod
-    def parseLineString(d, defaultcrs):
-        crs = d.get("crs", defaultcrs)
+    def parseLineString(self, d):
+        crs = d.get("crs", self.defaultcrs)
         return LineString(d["coordinates"], crs)
 
-    @staticmethod
-    def parseMultiLineString(d, defaultcrs):
-        crs = d.get("crs", defaultcrs)
+    def parseMultiLineString(self, d):
+        crs = d.get("crs", self.defaultcrs)
         return MultiLineString(d["coordinates"], crs)
 
-    @staticmethod
-    def parsePolygon(d, defaultcrs):
-        crs = d.get("crs", defaultcrs)
+    def parsePolygon(self, d):
+        crs = d.get("crs", self.defaultcrs)
         return Polygon(d["coordinates"], crs)
 
-    @staticmethod
-    def parseMultiPolygon(d, defaultcrs):
-        crs = d.get("crs", defaultcrs)
+    def parseMultiPolygon(self, d):
+        crs = d.get("crs", self.defaultcrs)
         return MultiPolygon(d["coordinates"], crs)
 
-    def parseGeometry(self, o, defaultcrs):
-        crs = o.get("crs", defaultcrs)
+    def parseGeometry(self, o):
         t = o["type"]
         if t == "Point":
-            return self.parsePoint(o, crs)
+            return self.parsePoint(o)
         elif t == "MultiPoint":
-            return self.parseMultiPoint(o, crs)
+            return self.parseMultiPoint(o)
         elif t == "LineString":
-            return self.parseLineString(o, crs)
+            return self.parseLineString(o)
         elif t == "MultiLineString":
-            return self.parseMultiLineString(o, crs)
+            return self.parseMultiLineString(o)
         elif t == "Polygon":
-            return self.parsePolygon(o, crs)
+            return self.parsePolygon(o)
         elif t == "MultiPolygon":
-            return self.parseMultiPolygon(o, crs)
+            return self.parseMultiPolygon(o)
         else:
             raise TypeError("Unrecognized type {0}".format(t))
 
-    def parseGeometryCollection(self, o, defaultcrs):
-        crs = o.get("crs", defaultcrs)
-        geoms = [self.parseGeometry(g) for g in o["geometries"]]
+    def parseGeometryCollection(self, o):
+        crs = o.get("crs", self.defaultcrs)
+        geoms = [self.parse(g) for g in o["geometries"]]
         return GeometryCollection(geoms, crs)
 
-    def parseFeature(self, o, defaultcrs):
-        crs = o.get("crs", defaultcrs)
-        geom = self.parseGeometry(o["geometry"], crs)
-        prop = self.parseProperties(o["properties"], len(geom.coordinates))
+    def parseFeature(self, o):
+        crs = o.get("crs", self.defaultcrs)
+        geom = self.parse(o["geometry"])
+        if isinstance(geom, GeometryCollection):
+            n = 1
+        else:
+            n = len(geom.coordinates)
+        prop = self.parseProperties(o["properties"], n)
         fid = o.get("id", None)
         return Feature(geom, prop, fid, crs)
 
-    def parseFeatureCollection(self, o, defaultcrs):
-        crs = o.get("crs", defaultcrs)
-        features = [self.parseFeature(f, crs) for f in o["features"]]
+    def parseFeatureCollection(self, o):
+        crs = o.get("crs", self.defaultcrs)
+        features = [self.parseFeature(f) for f in o["features"]]
         return FeatureCollection(features, crs)
 
     @staticmethod
@@ -273,31 +312,128 @@ class GeoJSONReader(object):
                 d["scalar"][key] = value
         return d
 
-    def items(self, o=None):
-        if o is None:
-            o = self.jsondict
-        crs = o.get("crs", DEFAULTCRS)
-        items = []
-        if o["type"] == "GeometryCollection":
-            items.append(self.parseGeometryCollection(o, crs))
-        elif o["type"] == "FeatureCollection":
-            items.append(self.parseFeatureCollection(o, crs))
-        elif o["type"] == "Feature":
-            items.append(self.parseFeature(o, crs))
+    def parse(self, d=None):
+        if d is None:
+            d = self.jsondict
+        if d["type"] == "GeometryCollection":
+            res = self.parseGeometryCollection(d)
+        elif d["type"] == "FeatureCollection":
+            res = self.parseFeatureCollection(d)
+        elif d["type"] == "Feature":
+            res = self.parseFeature(d)
         else:
-            items.append(self.parseGeometry(o, crs))
-        return items
+            res = self.parseGeometry(d)
+        return res
 
+class GeoJSONSerializer(object):
+    """ Class for converting GeoJSON named tuples to GeoJSON.
 
-def list_rec(A):
-    """ Recursively convert nested iterables to nested lists """
+    Usage:
+
+        serializer = GeoJSONSerializer()
+        json_string = serializer.serialize(named_tuple)
+    """
+
+    def __init__(self):
+        self.enc = NumpyAwareJSONEncoder()
+        return
+
+    def serialize(self, geom):
+        return self.enc.encode(self.geometry_asdict(geom))
+
+    def geometry_asdict(self, geom):
+
+        if hasattr(geom, "properties"):
+            return self.feature_asdict(geom)
+
+        elif hasattr(geom, "geometries"):
+            return self.geometry_collection_asdict(geom)
+
+        elif hasattr(geom, "features"):
+            return self.feature_collection_asdict(geom)
+
+        elif isinstance(geom, MultiPoint):
+            return self._geometry_asdict(geom, "MultiPoint")
+
+        elif isinstance(geom, Point):
+            return self._geometry_asdict(geom, "Point")
+
+        elif isinstance(geom, MultiLineString):
+            return self._geometry_asdict(geom, "MultiLineString")
+
+        elif isinstance(geom, LineString):
+            return self._geometry_asdict(geom, "LineString")
+
+        elif isinstance(geom, MultiPolygon):
+            return self._geometry_asdict(geom, "MultiPolygon")
+
+        elif isinstance(geom, Polygon):
+            return self._geometry_asdict(geom, "Polygon")
+        else:
+            raise TypeError("cannot serialize type '{0}'".format(type(geom)))
+
+    @staticmethod
+    def crsdict(crs=None, urn=None, href="", linktype="proj4"):
+        """ Return a dictionary that can be serialized as a GeoJSON coordinate
+        system mapping using a *urn* or a *crs* instance.
+
+        In the case of a linked CRS, the link address and type can be specified
+        using the `href` and `linktype` keyword arguments.
+
+        For more details, see the GeoJSON specification at:
+        http://geojson.org/geojson-spec.html#coordinate-reference-system-objects
+        """
+        if urn is not None:
+            return {'type': 'name',
+                    'properties': {'name': urn}}
+        elif crs is not None:
+            if hasattr(crs, "jsonhref") and hasattr(crs, "jsontype"):
+                d = {'type': 'link',
+                     'properties': {'href': crs.jsonname,
+                                    'type': crs.jsontype}}
+            elif hasattr(crs, "jsonname"):
+                d = {'type': 'name',
+                     'properties': {'name': crs.jsonname}}
+            else:
+                d = {'type': 'link',
+                     'properties': {'href': href,
+                                    'type': linktype}}
+            return d
+        else:
+            raise ValueError("either a urn must be specified or a CRS object "
+                             "provided")
+        return
+
+    def _geometry_asdict(self, geom, name):
+        return {"type": name,
+                "coordinates": geom.coordinates,
+                "crs": self.crsdict(crs=geom.crs)}
+
+    def feature_asdict(self, feature):
+        return {"type": "Feature",
+                "geometry": self.geometry_asdict(feature.geometry),
+                "properties": feature.properties}
+
+    def geometry_collection_asdict(self, geometry_collection):
+        return {"type": "GeometryCollection",
+                "geometries": [self.geometry_asdict(g)
+                               for g in geometry_collection.geometries]}
+
+    def feature_collection_asdict(self, feature_collection):
+        return {"type": "FeatureCollection",
+                "features": [self.feature_asdict(f)
+                             for f in feature_collection.features]}
+
+def asfixedlist(A):
+    """ Recursively convert nested iterables or coordinates to nested lists at
+    fixed precision. """
     if hasattr(A, '__iter__'):
-        return [list_rec(el) for el in A]
+        return [asfixedlist(el) for el in A]
     else:
-        return A
+        return round(A, 6)
 
 def printGeometryCollection(gpobj_list, **kwargs):
-    """ Given an iterable that returns guppy objects, construct a GeoJSON
+    """ Given an iterable that returns geometry objects, construct a GeoJSON
     FeatureCollection string.
     """
     geometrylist = []
@@ -362,5 +498,3 @@ def geojson2csv(fin, fout):
             f.close()
 
     return
-
-
