@@ -1,14 +1,11 @@
 """
 GPX IO and manipulation (UNDER DEVELOPMENT)
 
-This is a rewrite of gpxparser.py, designed to fit better with guppy types.
-
 To do:
     - XML namespaces aren't really handled properly
     - metadata node not addressed
 """
 
-import sys
 import collections
 import xml.etree.ElementTree as ET
 from xml.etree.ElementTree import ElementTree, Element
@@ -19,9 +16,9 @@ Trkseg = collections.namedtuple("Trkseg", ["trkpts", "properties", "extensions"]
 Track = collections.namedtuple("Track", ["trksegs", "properties", "extensions"])
 Route = collections.namedtuple("Route", ["rtepts", "properties", "extensions"])
 
+ET.register_namespace("", "http://www.topografix.com/GPX/1/1")
 ns = "{http://www.topografix.com/GPX/1/1}"
 
-
 VALID_PROPERTIES = ("ele", "time", "magvar", "geoidheight", "name", "cmt",
                     "desc", "src", "link", "sym", "type", "fix", "sat", "hdop",
                     "vdop", "pdop", "ageofdgpsdata", "dgpsid")
@@ -58,7 +55,8 @@ class GPX(object):
                     self.add_route(route)
         return
 
-    def _readextensions(self, node):
+    @staticmethod
+    def _readextensions(node):
         extensions = {}
         try:
             for ext in node.find(ns + "extensions"):
@@ -67,7 +65,8 @@ class GPX(object):
             pass
         return extensions
 
-    def _readproperties(self, node, exclude=()):
+    @staticmethod
+    def _readproperties(node, exclude=()):
         properties = {}
         for subnode in node:
             tag = strip_namespace(subnode.tag)
@@ -78,11 +77,12 @@ class GPX(object):
     def _readwpt(self, wpt):
         properties = self._readproperties(wpt, exclude=("extensions",))
         extensions = self._readextensions(wpt)
-        lon = wpt.attrib["lon"]
-        lat = wpt.attrib["lat"]
+        lon = round(float(wpt.attrib["lon"]), 6)
+        lat = round(float(wpt.attrib["lat"]), 6)
         return Point((lon, lat), properties, extensions)
 
-    def _dict2gpx(self, parent, properties):
+    @staticmethod
+    def _dict2gpx(parent, properties):
         for p in properties:
             sub = Element(ns + p)
             sub.text = str(properties[p])
@@ -143,7 +143,6 @@ class GPX(object):
 
         for node in self.gpx.findall(ns + "rte"):
             self.parse_rte(node)
-
         return
 
     def parse_wpt(self, wpt):
@@ -190,10 +189,9 @@ class GPX(object):
         self.waypts.append(waypt)
         return
 
-    def add_track(self, track, attributes=None):
-        """ Add a list of Line-like objects as a track. Dictionaries of
-        properties and extension types for the track are accepted as keyword
-        arguments.
+    def add_track(self, *tracks, **kw):
+        """ Add Line-like objects as a track. Dictionaries of properties and
+        extension types for the track are accepted as keyword arguments.
 
         Properties and extension types for the track segments are taken from
         the `properties` attribute of each Line-like object.
@@ -212,6 +210,7 @@ class GPX(object):
         properties = {}
         extensions = {}
 
+        attributes = kw.get("attributes", None)
         if attributes is not None:
             for key in attributes:
                 if key in VALID_PROPERTIES:
@@ -219,7 +218,7 @@ class GPX(object):
                 else:
                     properties[key] = str(attributes[key])
 
-        for line in track:
+        for line in tracks:
             points = []
             sg_properties = {}
             sg_extensions = {}
@@ -230,19 +229,23 @@ class GPX(object):
                 else:
                     sg_extensions[key] = str(line.properties[key])
 
-            pt_properties = [k for k in line.data.keys() if k in VALID_PROPERTIES]
-            pt_extensions = [k for k in line.data.keys() if k not in VALID_PROPERTIES]
+            if line.data is not None:
+                pt_properties = [k for k in line.data.fields if k in VALID_PROPERTIES]
+                pt_extensions = [k for k in line.data.fields if k not in VALID_PROPERTIES]
+            else:
+                pt_properties = []
+                pt_extensions = []
 
             for i, xy in enumerate(line.vertices):
 
                 ptprop = {}
                 ptexte = {}
                 for key in pt_properties:
-                    ptprop[key] = line.data[key][i]
+                    ptprop[key] = line.d[key][i]
                     if len(xy) == 3:
                         ptprop["ele"] = xy[2]
                 for key in pt_extensions:
-                    ptexte[key] = line.data[key][i]
+                    ptexte[key] = line.d[key][i]
 
                 points.append(Point((xy[0], xy[1]), ptprop, ptexte))
 
@@ -269,14 +272,15 @@ class GPX(object):
         points = []
         for i, vertex in enumerate(route.vertices):
             prop = {}
-            for k in route.data.keys():
-                prop[k] = route.data[k][i]
+            if route.data is not None:
+                for k in rout.data.fields:
+                    prop[k] = route.data[k][i]
             points.append(Point((vertex[0], vertex[1]), prop, {}))
         self.routes.append(Route(points, route.properties, {}))
         return
 
-    def writefile(self, fnm, waypts=True, tracks=True, routes=True):
-        """ Write GPX object to a GPX file. Writes all waypoints, tracks, and
+    def as_string(self, waypts=True, tracks=True, routes=True):
+        """ Write GPX object to a string. Writes all waypoints, tracks, and
         routes by default, which can be changed by changing the kwargs to
         False. """
         gpx = Element(ns + "gpx", version="1.1", creator="karta")
@@ -291,9 +295,11 @@ class GPX(object):
             for route in self.routes:
                 gpx.append(self._build_gpx_rte(route))
 
-        xmlstring = ET.tostring(gpx)
-        output = minidom.parseString(xmlstring).toprettyxml(indent="  ")
+        xmlstring = ET.tostring(gpx).decode("ascii")
+        return xmlstring
+
+    def writefile(self, fnm, **kw):
         with open(fnm, "w") as f:
-            f.write(output)
+            f.write(self.as_string(**kw))
         return
 
