@@ -1,9 +1,6 @@
-""" Implements a simple quadtree datastructure, with emphasis on performance. """
+""" Implements a simple QuadTree datastructure. """
 
-try:
-    from ._cvectorgeo import iswithin, hashpt
-except ImportError:
-    from ._vectorgeo import iswithin, hashpt
+from ._cvectorgeo import iswithin, hashpt, bbox_intersection_area
 
 class Node(object):
     def __init__(self, children, bbox, leaf):
@@ -13,14 +10,16 @@ class Node(object):
 
 class QuadTree(object):
     """ Implements a convenience class that wraps a quadtree data structure and
-    the methods needs to grow or query it. 
-    
-    Initialize with bbox (xmin, xmax, ymin, ymax).
+    the methods needed to grow or query it.
 
-    Optional parameters
-    -------------------
-    maxchildren         <int> maximum number of points before a node is split
-    maxdepth            <int> maximum tree depth
+    Parameters
+    ----------
+    bbox : 4-tuple
+        (xmin, xmax, ymin, ymax).
+    maxchildren : int, optional
+        maximum number of points before a node is split
+    maxdepth : int, optional
+        maximum tree depth
     """
 
     def __init__(self, bbox, maxchildren=20, maxdepth=999):
@@ -53,12 +52,29 @@ class QuadTree(object):
 
 
 def addpt(node, pt, depth, maxchildren, maxdepth):
-    """ Add *pt* to *node*. *depth* is the current level in the quadtree.
-    Returns the updated node (which may or may not be the same as the starting
-    node, depending on whether a split occurred, as well as the depth at which
-    the point was inserted. """
+    """ Add point to a node.
+
+    Parameters
+    ----------
+    node : Node
+    pt : tuple
+    depth : int
+        current quadtree level
+    maxchildren : int
+    maxdepth : int
+
+    Returns
+    -------
+    Node
+        updated node (which may or may not be the same as the starting node,
+        depending on whether a split occurred)
+    int
+        depth at which point was inserted
+    """
     if not iswithin(node.bbox, pt):
-        raise BBoxError
+        raise BBoxError("({x},{y}) is not within bounds {bbox}".format(x=pt[0],
+                                                                       y=pt[1],
+                                                                       bbox=node.bbox))
 
     if node.leaf:
 
@@ -86,25 +102,19 @@ def mean(x):
     else:
         None
 
-def overlaps(bbox0, bbox1):
-    pts0 = ((x, y) for x in bbox0[:2] for y in bbox0[2:])
-    if any(iswithin(bbox1, pt) for pt in pts0):
-        return True
-    pts1 = ((x, y) for x in bbox1[:2] for y in bbox1[2:])
-    if any(iswithin(bbox0, pt) for pt in pts1):
-        return True
-    return False
+def overlaps(bb0, bb1):
+    return bbox_intersection_area(bb0, bb1) != 0.0
 
 def split(node):
     """ Return a new node with the bbox of *node* and *node*'s children split
     between four child nodes. """
-    (x0, x1, y0, y1) = node.bbox
+    (x0, y0, x1, y1) = node.bbox
     xm = 0.5 * (x0+x1)
     ym = 0.5 * (y0+y1)
 
     branch = Node([], node.bbox, False)
-    bboxes = ((x0, xm, y0, ym), (xm, x1, y0, ym),
-              (x0, xm, ym, y1), (xm, x1, ym, y1))
+    bboxes = ((x0, y0, xm, ym), (xm, y0, x1, ym),
+              (x0, ym, xm, y1), (xm, ym, x1, y1))
 
     for bbox in bboxes:
         pts = [pt for pt in node.children if iswithin(bbox, pt)]
@@ -145,5 +155,7 @@ def querypt_hash(parent, pt):
             parent = parent.children[quad]
 
 class BBoxError(Exception):
-    pass
-
+    def __init__(self, message):
+        self.message = message
+    def __repr__(self):
+        return self.message
